class OrderController {

    def orderService

    def index = {
        if (!session.order) {
            session.order = new Order()
            session.order.customer = new Customer()
        }
        redirect(action:"search")
    }

    def goSearch = {
        redirect(action:"search")
    }

    def goCart = {
        redirect(action:"cart")
    }

    def searchFlow = {
        // 検索
        search {
            on("doSearch") {
                def searchResult = orderService.search(params)
                ["searchResult" : searchResult, "title" : params.title,
                 "author" : params.author, "isbn13" : params.isbn13,
                 "publisher" : params.publisher, "toCart" : false]
            }.to "search"
            on("detail") {
                def book = Book.get(params.id)
                ["book" : book, "toCart" : false]
            }.to "displayDetail"
            on("toCart") {
                orderService.putIntoCart(params, session.order)
                ["toCart" : true]
            }.to "search"
        }

        // 詳細
        displayDetail {
            on("search") {
                ["toCart" : false]
            }.to "search"
            on("toCart") {
                orderService.putIntoCart(params, session.order)
                ["toCart" : true]
            }.to "displayDetail"
        }
    }

    def cartFlow = {
        start {
            action {
                def order = session.order
                def totalPrice = orderService.calcTotalPrice(order)
                ["order" : order, "totalPrice" : totalPrice]
            }
            on("success").to "showCart"
        }

        // カートの中を見る
        showCart {
            on("editCart").to "editCart"
            on("goCredit") {
                redirect(action:"credit")
            }.to "showCart"
        }

        // カートの中を編集する
        editCart {
            on("submit") {
                def order = session.order
                if (!orderService.editCart(params, order)) {
                    return error()
                }
                def totalPrice = orderService.calcTotalPrice(order)
                ["order" : order, "totalPrice" : totalPrice]
            }.to "showCart"
            on("cancel").to "showCart"
        }
    }

    def creditFlow = {
        start {
            action {
                def order = session.order
                def totalPrice = orderService.calcTotalPrice(order)
                ["order" : order, "totalPrice" : totalPrice]
            }
            on("success").to "creditCartConfirm"
        }

        // カート内容確認
        creditCartConfirm {
            on("next") {
                def order = session.order
                ["order" : order]
            }.to "creditCustomerConfirm"
        }

        // 顧客情報入力
        creditCustomerConfirm {
            on("next") {
                def order = session.order
                order.customer = new Customer(params)
                order.customer.order = order

                if (!order.customer.validate()) {
                    return error(["order" : order, "customer" : order.customer])
                }

                ["order" : order]
            }.to "creditFinalConfirm"
            on("back") {
                def order = session.order
                order.customer = new Customer(params)
                order.customer.order = order

                ["order" : order]
            }.to "creditCartConfirm"
        }

        // 最終確認
        creditFinalConfirm {
            on("next") {
                def order = session.order

                // 注文日を設定
                order.date = new Date()

                // 注文、顧客情報を保存
                if (!order.save()) {
                    return error(["order" : order])
                }
                session.order = null
            }.to "creditComplete"
            on("back") {
                def order = session.order
                ["order" : order]
            }.to "creditCustomerConfirm"
        }

        creditComplete {
        }
    }
}
